@@ -4,6 +4,7 @@
 */
 
 #include	"courier.h"
+#include	"courier_lib_config.h"
 #include	"sbindir.h"
 #include	<string.h>
 #include	<stdio.h>
@@ -51,8 +52,8 @@ int	pipefd[2];
 		}
 		if (p == 0)
 		{
-			execl(SBINDIR "/courierlogger",
-			      "courierlogger", progname, (char *)0);
+			execl(COURIERLOGGER,
+			      COURIERLOGGER, progname, (char *)0);
 			_exit(5);
 		}
 		_exit(0);
