@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -38,7 +38,10 @@
 #include	"numlib/numlib.h"
 #include	"tcpd/spipe.h"
 #include	"tcpd/tlsclient.h"
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.45 2004/08/08 22:01:47 mrsam Exp $";
+#include	<courierauth.h>
+#include	<courierauthsasl.h>
+
+static const char rcsid[]="$Id: courieresmtpd.c,v 1.51 2005/12/13 03:00:54 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
@@ -72,15 +75,10 @@ static void tarpit()
 		teergrube=MAX_TEERGRUBE;
 }
 
-void addiovec_error(const char *q, const char *p)
+void iov_logerror(const char *q, const char *p)
 {
-const char *ident=getenv("TCPREMOTEINFO");
+	const char *ident=getenv("TCPREMOTEINFO");
 
-	if (q)
-		addiovec(q, strlen(q));
-
-	addiovec(p, strlen(p));
-	addiovec("\r\n", 2);
 	clog_msg_start_info();
 	clog_msg_str("error,relay=");
 	clog_msg_str(tcpremoteip);
@@ -93,11 +91,18 @@ const char *ident=getenv("TCPREMOTEINFO");
 	clog_msg_str(",msg=\"");
 	if (q)
 		clog_msg_str(q);
-
 	clog_msg_str(p);
 	clog_msg_str("\",cmd: ");
 	clog_msg_str(input_line);
 	clog_msg_send();
+}
+
+void addiovec_error(const char *p)
+{
+	addiovec(p, strlen(p));
+	addiovec("\r\n", 2);
+
+	iov_logerror(NULL, p);
 	iovflush();
 
 	if (*p == '5')
@@ -121,7 +126,7 @@ const char *p;
 	} while (*banner);
 }
 
-static void ehlo(const char *heloname, int hasauth, int hastls, int tls,
+static void ehlo(const char *heloname, int hastls, int tls,
 		 int extended)
 {
 static const char e[]=
@@ -134,13 +139,14 @@ EHLO_SECURITY_EXTENSION
 "250-SIZE\r\n"
 "250 DSN\r\n";
 const char *me=config_me();
+const char *p;
 
-	helobuf[0]=0;
-	strncat(helobuf, heloname, sizeof(helobuf)-1);
-
+	if (helobuf[0] == 0) 
 	{
 		char *p;
 
+		strncat(helobuf, heloname, sizeof(helobuf)-1);
+
 		for (p=helobuf; *p; p++)
 			if (isspace((int)(unsigned char)*p) ||
 			    (int)(unsigned char)*p < ' ')
@@ -158,32 +164,28 @@ const char *me=config_me();
 	addiovec("250-", 4);
 	addiovec(me, strlen(me));
 	addiovec(" Ok.\r\n", 6);
-	if (hasauth)
+
+	if (tls && (p=getenv("ESMTPAUTH_TLS")) != 0 && *p)
+		;
+	else
+		p=getenv("ESMTPAUTH");
+
+	if (p && *p)
 	{
-		const char *p;
+		addiovec("250-AUTH ", 9);
+		addiovec(p, strlen(p));
+		addiovec("\r\n", 2);
 
-		if (tls && (p=getenv("ESMTPAUTH_TLS")) != 0 && *p)
-			;
-		else
-			p=getenv("ESMTPAUTH");
+		/*
+		** Netscape.
+		*/
 
-		if (p && *p)
-		{
-			addiovec("250-AUTH ", 9);
-			addiovec(p, strlen(p));
-			addiovec("\r\n", 2);
+		addiovec("250-AUTH=", 9);
+		addiovec(p, strlen(p));
+		addiovec(" X-NETSCAPE-HAS-BUGS\r\n", 22);
 
-	/*
-	** The following shit is necessary because Netscape developers don't
-	** know how to read the fucking RFCs.
-	*/
-
-			addiovec("250-AUTH=", 9);
-			addiovec(p, strlen(p));
-			addiovec(" X-NETSCAPE-HAS-BUGS\r\n", 22);
-
-		}
 	}
+
 	if (hastls)
 	{
 		static const char starttls_msg[]=
@@ -199,7 +201,7 @@ extern char **environ;
 
 static void badfork()
 {
-	addiovec_error(NULL, "432 Service temporarily unavailable.");
+	addiovec_error("432 Service temporarily unavailable.");
 }
 
 static void expnvrfy(const char *line, const char *cmd)
@@ -217,7 +219,7 @@ int	rc;
 
 	if (a->naddrs != 1 || a->addrs[0].tokens == 0)
 	{
-		addiovec_error(NULL, "502 EXPN syntax error.");
+		addiovec_error("502 EXPN syntax error.");
 		rfc822a_free(a);
 		rfc822t_free(t);
 		return;
@@ -252,7 +254,7 @@ int	rc;
 
 static void startsubmit(int tls)
 {
-char	*argv[10];
+char	*argv[13];
 const char *ident;
 char	*identbuf=0;
 int	n;
@@ -263,10 +265,25 @@ char	*buf;
 	if (helobuf[0] == '\0')	return;
 
 	argv[0]="submit";
-	argv[1]=NAME;
-
+	argv[1]=getenv("RELAYCLIENT") ? "-src=authsmtp":"-src=smtp";
 	n=2;
 
+	if (authuserbuf[0])
+	{
+		char *p;
+
+		static char authbuf[sizeof(authuserbuf)+sizeof("-auth=")];
+
+		p=strchr(authuserbuf, ' ');
+		if (p)
+		{
+			strcat(strcpy(authbuf, "-auth="), p+1);
+			argv[n++]=authbuf;
+		}
+	}
+
+	argv[n++]="esmtp";
+
 	host=tcpremotehost;
 
 	if (!host)	host="";
@@ -289,20 +306,21 @@ char	*buf;
 						strlen(tlsbuf)+
 						strlen(ident)+
 						strlen(authuserbuf));
+		++n;
 		*identbuf=0;
 		if (*ident)
 			strcat(strcat(identbuf, "IDENT: "), ident);
-		if (authuserbuf[0])
-		{
-			if (*identbuf)	strcat(identbuf, ", ");
-			strcat(strcat(identbuf, "AUTH: "), authuserbuf);
+                if (authuserbuf[0])
+                {
+                        if (*identbuf)  strcat(identbuf, ", ");
+                        strcat(strcat(identbuf, "AUTH: "), authuserbuf);
 		}
+
 		if (tls)
 		{
 			if (*identbuf)	strcat(identbuf, ", ");
 			strcat(identbuf, tlsbuf);
 		}
-		++n;
 	}
 
 	argv[n]=0;
@@ -444,7 +462,7 @@ const char *q=skipaddress(&p);
 		set_submit_error(0, 0);
 		return (domailfrom(p, q));
 	}
-	addiovec_error(NULL, "554 Syntax error - your mail software violates RFC 821.");
+	addiovec_error("554 Syntax error - your mail software violates RFC 821.");
 	return (-1);
 }
 
@@ -542,7 +560,7 @@ int	rc;
 					(n += *r-'0') < l ||
 					(sizelimit && n > sizelimit))
 				{
-					addiovec_error(NULL, "534 SIZE=Message too big.");
+					addiovec_error("534 SIZE=Message too big.");
 					return (-1);
 				}
 				l=n;
@@ -607,7 +625,7 @@ const char *q=skipaddress(&p);
 		set_submit_error(p-1, q-p+2);
 		return (dorcptto(p, q));
 	}
-	addiovec_error(NULL, "554 Syntax error - your mail software violates RFC 821.");
+	addiovec_error("554 Syntax error - your mail software violates RFC 821.");
 	return (-1);
 }
 
@@ -872,6 +890,41 @@ const char *p=getenv(bofh);
 	return (0);
 }
 
+static char *sasl_conv_func(const char *s, void *dummy)
+{
+char	*p;
+
+	addiovec("334 ", 4);
+	addiovec(s, strlen(s));
+	addiovec("\r\n", 2);
+	iovflush();
+
+	p=iovreadline();
+	if (!p)	exit(0);
+	if ((p=strdup(p)) == 0)
+	{
+		perror("malloc");
+		exit(0);
+	}
+	return (p);
+}
+
+static int auth_callback_func(struct authinfo *a, void *va)
+{
+	char *p;
+
+	strcpy(authuserbuf, "");
+	strncat(authuserbuf, (const char *)va, 20);
+	strcat(authuserbuf, " ");
+	strncat(authuserbuf, a->address,
+		sizeof(authuserbuf)-10-strlen(a->address));
+
+	for (p=authuserbuf; *p; p++)
+		if ((int)(unsigned char)*p < ' ' || *p >= 127)
+			*p=' ';
+	return 0;
+}
+
 /*****************************************************************************
 
 courieresmtpd can be started in two situations:
@@ -913,6 +966,7 @@ int	authenticated=0;
 		exit(1);
 	}
 
+	helobuf[0]=0;
 	submit_set_teergrube(tarpit);
 
 	if (strcmp(argv[0], "sendmail") == 0)
@@ -952,7 +1006,7 @@ int	authenticated=0;
 				tls_required=1;
 				if (!starttls)
 				{
-					addiovec_error(NULL, "440 TLS not available,  but"
+					addiovec_error("440 TLS not available,  but"
 						" it's required for this connection.");
 					return (1);
 				}
@@ -987,7 +1041,7 @@ int	authenticated=0;
 	signal(SIGPIPE, SIG_IGN);
 	/* Some house keeping, while waiting for a reply */
 
-	if (rw_init(NAME))	return (1);
+	if (rw_init("esmtp"))	return (1);
 
 	clog_msg_start_info();
 	clog_msg_str("started,ip=[");
@@ -1028,7 +1082,7 @@ int	authenticated=0;
 			strncmp(line, "HELO ", 5) == 0) &&
 			line[5])
 		{
-			ehlo(line+5, argc > 1, starttls, tls, line[0] == 'E');
+			ehlo(line+5, starttls, tls, line[0] == 'E');
 			iovflush();
 			cancelsubmit();
 			startsubmit(tls);
@@ -1045,12 +1099,12 @@ int	authenticated=0;
 
 			if (!starttls)
 			{
-				addiovec_error(NULL, "540 TLS not available.");
+				addiovec_error("540 TLS not available.");
 				continue;
 			}
 			if (libmail_streampipe(pipefd))
 			{
-				addiovec_error(NULL, "432 libmail_streampipe() failed.");
+				addiovec_error("432 libmail_streampipe() failed.");
 				exit(1);
 			}
 
@@ -1110,20 +1164,19 @@ int	authenticated=0;
 
 		if (tls_required && strcmp(line, "RSET"))
 		{
-			addiovec_error(NULL, "400 STARTTLS is required first.");
+			addiovec_error("400 STARTTLS is required first.");
 			continue;
 		}
 
 		if (strncmp(line, "AUTH ", 5) == 0 && mailfroms == 0
-			&& argc > 1  && authuserbuf[0] == 0)
+			&& authuserbuf[0] == 0)
 		{
 		char	*authmethod;
 		char	*initreply;
-		pid_t	p;
-		int	pipefd[2];
-		int	i, j;
 		const char *q;
 		char	*buf=strcpy(courier_malloc(strlen(line)+1), line);
+		char	*authtype;
+		char	*authdata;
 
 			starttls=0;
 			(void)strtok(buf, " \t\r\n");
@@ -1139,73 +1192,40 @@ int	authenticated=0;
 
 			if (authmethod == 0 || *authmethod == 0)
 			{
-				addiovec_error(NULL, "535 Authentication rejected");
+				addiovec_error("535 Authentication rejected");
 				free(buf);
 				continue;
 			}
 
-			signal(SIGCHLD, SIG_DFL);
 
-			if (pipe(pipefd) < 0 || (p=fork()) < 0)
+			if (auth_sasl(authmethod, initreply && *initreply
+				      ? initreply:NULL,
+				      sasl_conv_func,
+				      NULL,
+				      &authtype,
+				      &authdata) != AUTHSASL_OK)
 			{
-				addiovec_error(NULL, "535 Unable to fork - try again later.");
-				exit(1);
+				/* TODO - better error messages */
+
+				strcpy(authuserbuf, "");
+			}
+			else
+			{
+				int rc=auth_generic("esmtp", authtype,
+						    authdata,
+						    auth_callback_func,
+						    authmethod);
+
+				free(authtype);
+				free(authdata);
+
+				if (rc)
+					strcpy(authuserbuf, "");
 			}
 
-			if (p == 0)
-			{
-			char	*q;
-
-				/*
-				** authstart wants the write end of the
-				** pipe to be on file descriptor 3.
-				*/
-
-				close(pipefd[0]);
-				if (pipefd[1] != 3)
-				{
-					close(3);
-					if (dup(pipefd[1]) != 3)
-						exit (0);
-					close(pipefd[1]);
-				}
-
-				q=malloc(sizeof("ESMTPAUTHMETHOD=")+
-					strlen(authmethod));
-				if (!q)	exit(0);
-				strcat(strcpy(q, "ESMTPAUTHMETHOD="),
-					authmethod);
-				if (putenv(q) < 0)	exit(0);
-
-				if (initreply && *initreply)
-				{
-					q=malloc(sizeof("ESMTPINITREPLY=")+
-						strlen(initreply));
-					strcat(strcpy(q, "ESMTPINITREPLY="),
-						initreply);
-					if (putenv(q) < 0)	exit(0);
-				}
-				execv(argv[1], argv+1);
-				exit(0);
-			}
-			close(pipefd[1]);
-			i=0;
-			while (i < sizeof(authuserbuf)-1 &&
-				(j=read(pipefd[0], authuserbuf+i,
-					sizeof(authuserbuf)-1-i)) > 0)
-				i += j;
-			authuserbuf[i]=0;
-			for (i=0; authuserbuf[i]; i++)
-				if (authuserbuf[i] == '\r' ||
-					authuserbuf[i] == '\n')
-					break;
-			authuserbuf[i]=0;
-			close(pipefd[0]);
-			while (wait(&i) != p)
-				;
 			if (authuserbuf[0] == 0)
 			{
-				addiovec_error(NULL, "535 Authentication failed.");
+				addiovec_error("535 Authentication failed.");
 				iovflush();
 			}
 			else
@@ -1233,7 +1253,7 @@ int	authenticated=0;
 
 		if (auth_required && !authenticated && strcmp(line, "RSET"))
 		{
-			addiovec_error(NULL, "535 Authentication required.");
+			addiovec_error("535 Authentication required.");
 			continue;
 		}
 
@@ -1246,7 +1266,7 @@ int	authenticated=0;
 				expnvrfy(line+5, "-expn=");
 				startsubmit(tls);
 			}
-			else	addiovec_error(NULL, "252 EXPN disabled");
+			else	addiovec_error("252 EXPN disabled");
 			continue;
 		}
 		if (strncmp(line, "VRFY ", 5) == 0)
@@ -1258,7 +1278,7 @@ int	authenticated=0;
 				expnvrfy(line+5, "-vrfy=");
 				startsubmit(tls);
 			}
-			else	addiovec_error(NULL, "252 VRFY disabled");
+			else	addiovec_error("252 VRFY disabled");
 			continue;
 		}
 		if (strcmp(line, "RSET") == 0)
@@ -1286,7 +1306,7 @@ int	authenticated=0;
 			starttls=0;
 			if (!helobuf[0])
 			{
-				addiovec_error(NULL, "502 Polite people say HELO first");
+				addiovec_error("502 Polite people say HELO first");
 				continue;
 			}
 			if (!submit_started)
@@ -1329,7 +1349,7 @@ int	authenticated=0;
 			startsubmit(tls);
 			continue;
 		}
-		addiovec_error(NULL, "502 ESMTP command error");
+		addiovec_error("502 ESMTP command error");
 	}
 	addiovec("221 Bye.\r\n", 10);
 	iovflush();
