@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -22,9 +22,9 @@
 #include	"rfc822.h"
 #include	"rfc2045/rfc2045.h"
 #include	"numlib/numlib.h"
-#include	"authlib/authsaslclient.h"
 #include	"tcpd/spipe.h"
 #include	"tcpd/tlsclient.h"
+#include	<courierauthsaslclient.h>
 
 #define	mybuf_readfunc	sox_read
 #include	"mybuf.h"
@@ -140,8 +140,9 @@ struct	ctlfile	ctf;
 unsigned long mypid=(unsigned long)getpid();
 
 	signal(SIGPIPE, SIG_IGN);
-	rw_init(NAME);
-	rewrite_func=rw_search_transport(NAME)->rw_ptr->rewrite;
+	srand(time(NULL));
+	rw_init("esmtp");
+	rewrite_func=rw_search_transport("esmtp")->rw_ptr->rewrite;
 
 	if (chdir(courierdir()))
 		clog_msg_errno();
@@ -176,14 +177,14 @@ unsigned long mypid=(unsigned long)getpid();
 	struct	timeval	tv;
 	const char *p;
 
-/*
+#if 0
 		clog_msg_start_info();
 		clog_msg_str("Process ");
 		clog_msg_uint(getpid());
 		clog_msg_str(" ready to be grabbed.");
 		clog_msg_send();
 		sleep(60);
-*/
+#endif
 
 		/*
 		** Open the message control file, send the message, close
@@ -305,11 +306,32 @@ static int get_sourceaddr(int af, RFC1035_NETADDR *addrbuf,
 	return 0;
 }
 
+static int backscatter(const char *src)
+{
+	char buf[1024];
+	const char *env;
+	char *p;
+
+	buf[0]=0;
+
+	env=getenv("ESMTP_BLOCKBACKSCATTER");
+
+	strncat(buf, env ? env:"", 1000);
+
+	for (p=buf; (p=strtok(p, ",")); p=NULL)
+	{
+		if (strcmp(p, src) == 0)
+			return 1;
+	}
+	return 0;
+}
+
 /* Attempt to deliver a message */
 
 static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 {
 	char *smtproute;
+	int cn;
 
 	if (!host)
 		host=strcpy(courier_malloc(strlen(del->host)+1), del->host);
@@ -354,6 +376,36 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 		net_timeout=0;
 	}
 
+	if ((cn=ctlfile_searchfirst(ctf, COMCTLFILE_MSGSOURCE)) >= 0 &&
+	    backscatter(ctf->lines[cn]+1))
+	{
+		int i;
+
+		for (i=0; i<del->nreceipients; i++)
+		{
+			ctlfile_append_connectioninfo(ctf,
+						      (unsigned)
+						      atol(del->receipients
+							   [i*2]),
+						      COMCTLFILE_DELINFO_REPLYTYPE,
+						      "smtp");
+
+			ctlfile_append_connectioninfo(ctf,
+						      (unsigned)
+						      atol(del->receipients
+							   [i*2]),
+						      COMCTLFILE_DELINFO_REPLY,
+						      "250 Backscatter bounce dropped.");
+
+			ctlfile_append_reply(ctf,
+					     (unsigned)atol( del->receipients[i*2]),
+					     "delivered: backscatter bounce dropped",
+					     COMCTLFILE_DELSUCCESS_NOLOG,
+					     (hasdsn ? "":" r"));
+		}
+		return;
+	}
+
 	/*
 	** If the message wants a secured connection, and the current
 	** connection has not been secured, close it, so it can be reopened.
@@ -506,6 +558,7 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 				sox_close(sockfd);
 			sockfd= -1;
 
+#if 0
 			if (p->next && p->priority == p->next->priority &&
 				strcmp(p->hostname, p->next->hostname) == 0)
 			{
@@ -515,6 +568,7 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 			/* Skip other MX records with the same priority */
 			while (p->next && p->priority == p->next->priority)
 				p=p->next;
+#endif
 		}
 
 		rfc1035_mxlist_free(mxlist);
@@ -997,11 +1051,24 @@ static void burp(const char *p, unsigned n)
 	Try EHLO then HELO, and see what the other server says.
 */
 
+static RFC1035_ADDR laddr;
+
 static int hello2(struct moduledel *, struct ctlfile *, int);
 
 static int hello(struct moduledel *del, struct ctlfile *ctf)
 {
 	const char *p;
+	RFC1035_NETADDR lsin;
+	socklen_t i;
+
+	i=sizeof(lsin);
+	if (sox_getsockname(sockfd, (struct sockaddr *)&lsin, &i) ||
+	    rfc1035_sockaddrip(&lsin, i, &laddr))
+	{
+		soft_error1(del, ctf, "Cannot obtain local socket IP address.",
+			    -1);
+		return -1;
+	}
 
 	mybuf_init(&sockbuf, sockfd);
 	writebufptr=writebuf;
@@ -1065,14 +1132,40 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 {
 	const	char *p;
 	char	hellobuf[512];
+	char buf[RFC1035_MAXNAMESIZE+128];
 
 	haspipelining=hasdsn=has8bitmime=hasverp=hassize=hasexdata=hascourier=
 	hasstarttls=hassecurity_starttls=0;
 	if (authsasllist)	free(authsasllist);
 	authsasllist=0;
 
+	p=config_esmtphelo();
+
+	/*
+	** If the remote host is "*", use reverse DNS from the local IP addr.
+	*/
+
+	if (strcmp(p, "*") == 0)
+	{
+		p=buf;
+		if (rfc1035_ptr(&rfc1035_default_resolver, &laddr, buf) != 0)
+		{
+			char *q;
+
+			rfc1035_ntoa(&laddr, buf+1);
+
+			q=buf+1;
+
+			if (strncmp(q, "::ffff:", 7) == 0)
+				q += 7;
+			*--q='[';
+			strcat(q, "]");
+			p=q;
+		}
+	}
+
 	strcpy(hellobuf, "EHLO ");
-	strncat(hellobuf, config_esmtphelo(), sizeof(hellobuf)-10);
+	strncat(hellobuf, p, sizeof(hellobuf)-10);
 	strcat(hellobuf, "\r\n");
 
 	if (dowritestr(hellobuf) || writeflush())
@@ -2274,7 +2367,7 @@ static int parseexdatareply(const char *p,
 	struct moduledel *del, struct ctlfile *ctf, int *rcptok)
 {
 unsigned i;
-char err_code;
+char err_code=0;
 unsigned line_num=0;
 
 	for (i=0; i<del->nreceipients; i++)
@@ -2625,7 +2718,7 @@ int	rc;
 	info.plain_conv_func= &plain_esmtpauth;
 	info.conv_func_arg= &xinfo;
 
-	rc=authsaslclient(&info);
+	rc=auth_sasl_client(&info);
 	if (rc == AUTHSASL_NOMETHODS)
 	{
 		talking(del, ctf);
