@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2003 Double Precision, Inc.
+** Copyright 1998 - 2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -54,7 +54,7 @@
 #include	"liblock/liblock.h"
 #include	"liblock/mail.h"
 
-static const char rcsid[]="$Id: deliver.c,v 1.21 2003/01/23 13:26:51 mrsam Exp $";
+static const char rcsid[]="$Id: deliver.c,v 1.23 2005/02/04 02:44:59 mrsam Exp $";
 
 extern char *local_dotcourier(const char *, const char *, const char **);
 extern char *local_extension();
@@ -134,7 +134,7 @@ char		*userhome;
 			"Mail loop - already have my Delivered-To: header.");
 		if (ctlfile)	free(ctlfile);
 		if (extname)	free(extname);
-		exit (EX_NOUSER);
+		exit (EX_SOFTWARE);
 	}
 
 	dodel(username, userhome, stdin,
@@ -394,8 +394,8 @@ static void delivery_error(const char *name)
 {
 	if (errno == ENOSPC)
 	{
-		fprintf(stderr, "Mailbox full.\n");
-		exit(EX_NOPERM);
+		fprintf(stderr, "Maildir over quota.\n");
+		exit(EX_TEMPFAIL);
 	}
 
 #if	HAVE_STRERROR
@@ -842,11 +842,14 @@ const char *shell=getenv("SHELL");
 					recursion_level+1);
 		}
 		if (newcommand)	free(newcommand);
+
+		if (rc == EX_SOFTWARE)
+			rc=EX_NOPERM;
 		return (rc);
 	}
 	if (newcommand)	free(newcommand);
 
-	return (EX_SOFTWARE);
+	return (EX_NOPERM);
 }
 
 static char *read_command(int fd)
