@@ -23,10 +23,9 @@
 #include	<stdio.h>
 #include	<pwd.h>
 #include	"waitlib/waitlib.h"
-#include	"authlib/auth.h"
-#include	"authlib/authstaticlist.h"
 #include	"modulelist.h"
 #include	"sysconfdir.h"
+#include	<courierauth.h>
 
 #if	HAVE_SYSLOG_H
 #include	<syslog.h>
@@ -36,7 +35,7 @@
 
 #define DEFAULTNAME	"alias@"
 
-static const char rcsid[]="$Id: local.c,v 1.21 2004/05/09 17:47:04 mrsam Exp $";
+static const char rcsid[]="$Id: local.c,v 1.23 2004/10/24 02:01:55 mrsam Exp $";
 
 extern char *local_dotcourier(const char *, const char *, const char **);
 
@@ -55,7 +54,7 @@ static int rw_local_filter(const char *,	/* Sending module */
 
 struct rw_list *local_rw_install(const struct rw_install_info *p)
 {
-static struct rw_list local_info={0, "module." NAME " - " COURIER_COPYRIGHT,
+static struct rw_list local_info={0, "module.local - " COURIER_COPYRIGHT,
 			 rw_local, rw_del_local, rw_local_filter};
 
 	return (&local_info);
@@ -111,10 +110,6 @@ static int cleanup_set=0;
 
 static void cleanup()
 {
-int	i;
-
-	for (i=0; authstaticmodulelist[i]; i++)
-		(*authstaticmodulelist[i]->auth_cleanupfunc)();
 }
 
 /*
@@ -141,8 +136,6 @@ struct	rfc822token	hostdomaint;
 #if LOCAL_EXTENSIONS
 char *atdomain;
 #endif
-const char *authmodules=getenv("AUTHMODULES");
-
 
 	if (!cleanup_set)
 	{
@@ -271,9 +264,8 @@ const char *authmodules=getenv("AUTHMODULES");
 		lai.rwi=rwi;
 		lai.found=0;
 
-		i=authstaticlist_search_string(search_addr, "courier",
-					       authmodules,
-					       &local_callback, &lai);
+		i=auth_getuserinfo("courier", search_addr,
+				   local_callback, &lai);
 
 #if LOCAL_EXTENSIONS
 		if (alloc_buf)
@@ -320,9 +312,9 @@ const char *authmodules=getenv("AUTHMODULES");
 				strcat(strcpy(alloc_buf, DEFAULTNAME),
 				       atdomain);
 
-				i=authstaticlist_search(alloc_buf, "courier",
-							MODULELIST,
-							&local_callback, &lai);
+				i=auth_getuserinfo("courier",
+						   alloc_buf,
+						   &local_callback, &lai);
 				free(alloc_buf);
 				if (i == 0)
 				{
@@ -411,7 +403,7 @@ not_found:
 
 	if ((rwi->mode & (RW_SUBMIT|RW_EXPN|RW_VERIFY)) == RW_SUBMIT &&
 	    rwi->smodule &&
-	    (strcmp(rwi->smodule, NAME) == 0 ||
+	    (strcmp(rwi->smodule, "local") == 0 ||
 	     strcmp(rwi->smodule, "uucp") == 0))
 	{
 		(*delfunc)(rwi, rwi->ptr, rwi->ptr);
