@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2003 Double Precision, Inc.
+** Copyright 1998 - 2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -35,10 +35,8 @@
 #include	"moduledel.h"
 #include	"comreadtime.h"
 #include	"waitlib/waitlib.h"
-#include	"userdb/userdb.h"
 #include	"numlib/numlib.h"
 #include	"comverp.h"
-#include	"auth.h"
 
 #if	HAVE_SYS_FILE_H
 #include	<sys/file.h>
@@ -54,6 +52,7 @@
 #define	EX_TEMPFAIL	75
 #define	EX_NOUSER	67
 #define	EX_CANTCREAT	73
+#define	EX_SOFTWARE	70
 #endif
 
 static const char *courier_home;
@@ -544,13 +543,22 @@ char	*oreceipients=0;
 					"Message delivered.",
 					COMCTLFILE_DELSUCCESS, " l");
 			}
+			if (oreceipients)
+				free (oreceipients);
 			return;
+		case EX_SOFTWARE:
+			ctlfile_append_reply(ctf, rcptnum,
+					"", COMCTLFILE_DELFAIL_NOTRACK, 0);
+			if (oreceipients)
+				free (oreceipients);
+			return;
+
+
 		case 64:
 		case 65:
 		case 67:
 		case 68:
 		case 69:
-		case 70:
 		case 76:
 		case 77:
 		case 78:
@@ -560,6 +568,8 @@ char	*oreceipients=0;
 		default:
 			ctlfile_append_reply(ctf, rcptnum,
 					"", COMCTLFILE_DELDEFERRED, 0);
+			if (oreceipients)
+				free (oreceipients);
 			return;
 		}
 
