@@ -3,7 +3,7 @@
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: gpg.c,v 1.5 2003/08/12 18:22:38 mrsam Exp $";
+static const char rcsid[]="$Id: gpg.c,v 1.10 2005/11/23 01:08:27 mrsam Exp $";
 
 #include "config.h"
 #include <stdio.h>
@@ -35,6 +35,7 @@ static const char rcsid[]="$Id: gpg.c,v 1.5 2003/08/12 18:22:38 mrsam Exp $";
 #include "mimegpgfork.h"
 #include "tempname.h"
 #include "gpglib.h"
+#include "rfc822/encode.h"
 
 static int my_rewind(FILE *fp)
 {
@@ -287,6 +288,7 @@ static int encode_header(const char *h)
 struct gpg_fork_output_info {
 	void (*output_func)(const char *, size_t, void *);
 	void *output_func_arg;
+	struct gpgmime_forkinfo *gpgptr;
 };
 
 static int gpg_fork_output(const char *p, size_t n, void *dummy)
@@ -362,6 +364,9 @@ static int dogpgencrypt(const char *gpghome,
 	  "\n"
 	  "This is a MIME GnuPG-encrypted message.  If you see this text, it means\n"
 	  "that your E-mail or Usenet software does not support MIME encrypted messages.\n"
+	  "The Internet standard for MIME PGP messages, RFC 2015, was published in 1996.\n"
+	  "To open this message correctly you will need to install E-mail or Usenet\n"
+	  "software that supports modern Internet standards.\n"
 	  "\n--");
 
 	S(boundary);
@@ -499,6 +504,37 @@ static int dogpgencrypt(const char *gpghome,
 	return 0;
 }
 
+static int qp_do(const char *p, size_t cnt, void *ptr)
+{
+	struct gpg_fork_output_info *gfoi=
+		(struct gpg_fork_output_info *)ptr;
+	struct gpgmime_forkinfo *gpg=gfoi->gpgptr;
+	size_t i;
+
+	(*gfoi->output_func)(p, cnt, gfoi->output_func_arg);
+
+	while (cnt)
+	{
+		if (*p == '\n')
+		{
+			libmail_gpgmime_write(gpg, "\r\n", 2);
+			++p;
+			--cnt;
+			continue;
+		}
+
+
+		for (i=0; i<cnt; i++)
+			if (p[i] == '\n')
+				break;
+
+		libmail_gpgmime_write(gpg, p, i);
+		p += i;
+		cnt -= i;
+	}
+	return 0;
+}
+
 static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 		     struct mimestack **stack, struct header *h, int *iseof,
 		     int (*input_func)(char *, size_t, void *vp),
@@ -513,11 +549,11 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 		     void *errhandler_arg)
 {
 	struct header *hp;
-	char buf[BUFSIZ];
+	char buf[8192];
 	struct gpgmime_forkinfo gpg;
 	int clos_flag=0;
 	struct mimestack *b=0;
-	int rc;
+	int rc=0;
 	char signed_content_name[TEMPNAMEBUFSIZE];
 	int signed_content;
 	FILE *signed_content_fp;
@@ -526,6 +562,7 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 	int convert_to_qp=0;
 	int in_headers;
 	struct gpg_fork_output_info gfoi;
+	struct libmail_encode_info encode_info;
 
 	for (hp=h; hp; hp=hp->next)
 	{
@@ -659,6 +696,9 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 	  "\n"
 	  "This is a MIME GnuPG-signed message.  If you see this text, it means that\n"
 	  "your E-mail or Usenet software does not support MIME signed messages.\n"
+	  "The Internet standard for MIME PGP messages, RFC 2015, was published in 1996.\n"
+	  "To open this message correctly you will need to install E-mail or Usenet\n"
+	  "software that supports modern Internet standards.\n"
 	  "\n--");
 	S(boundary);
 	C("\n");
@@ -666,6 +706,7 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 
 	gfoi.output_func=output_func;
 	gfoi.output_func_arg=output_func_arg;
+	gfoi.gpgptr= &gpg;
 
 	if (libmail_gpgmime_forksignencrypt(gpghome, passphrase_fd,
 					    GPG_SE_SIGN,
@@ -677,6 +718,12 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 		return (-1);
 	}
 
+	if (convert_to_qp)
+		libmail_encode_start(&encode_info,
+				     "quoted-printable",
+				     qp_do,
+				     &gfoi);
+
 	in_headers=1;
 
 	while (fgets(buf, sizeof(buf), signed_content_fp) != NULL)
@@ -700,59 +747,23 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 		}
 		else if (convert_to_qp && !in_headers)
 		{
-			size_t n=0;
+			size_t i;
 
 			for (p=buf; *p; )
 			{
-				size_t i, j, k;
-
-				if (n >= 76)
+				if (*p == '\r')
 				{
-					libmail_gpgmime_write(&gpg, "=\r\n", 3);
-					(*output_func)("=\n", 2,
-						       output_func_arg);
-					n=0;
-					continue;
-				}
-
-				for (i=0; p[i]; i++)
-				{
-					if (p[i] == '=' ||
-					    (p[i] < ' ' &&
-					     p[i] != '\r' && p[i] != '\n') ||
-					    p[i] >= 0x7F)
-						break;
-
-					if (i && p[i] != '\r' && p[i] != '\n'
-					    && n + i >= 76)
-						break;
-				}
-
-				if (i == 0)
-				{
-					char x[4];
-
-					sprintf(x, "=%02X",
-						(int)(unsigned char)*p);
-
-					libmail_gpgmime_write(&gpg, x, 3);
-					(*output_func)(x, 3, output_func_arg);
-
 					++p;
-					n += 3;
 					continue;
 				}
 
-				libmail_gpgmime_write(&gpg, p, i);
-				n += i;
-
-				for (j=k=0; j<i; j++)
-					if (p[j] != '\r')
-						p[k++]=p[j];
-
-				if (k)
-					(*output_func)(p, k, output_func_arg);
+				for (i=0; p[i]; ++i)
+					if (p[i] == '\r')
+						break;
 
+				if (rc == 0 &&
+				    libmail_encode(&encode_info, p, i))
+					rc= -1;
 				p += i;
 			}
 			continue;
@@ -769,6 +780,12 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 			(*output_func)(p, k, output_func_arg);
 	}
 
+	if (convert_to_qp)
+	{
+		if (rc == 0 && libmail_encode_end(&encode_info))
+			rc= -1;
+	}
+
 	C("\n--");
 	S(boundary);
 
@@ -779,7 +796,8 @@ static int dogpgsign(const char *gpghome, const char *passphrase_fd,
 #undef C
 #undef S
 
-	rc=libmail_gpgmime_finish(&gpg);
+	if (libmail_gpgmime_finish(&gpg))
+		rc= -1; /* TODO */
 
 	if (rc)
 	{
@@ -941,6 +959,8 @@ static int dosignencode(int dosign, int doencode, int dodecode,
 			    && (bv=libmail_mimeheader_getattr(mh, "boundary")) != 0
 
 			    && (doencode & LIBMAIL_GPG_ENCAPSULATE) == 0
+
+			    && !dosign
 			    )
 			{
 				struct header *p;
@@ -1979,7 +1999,14 @@ static int dodecrypt(const char *gpghome,
 
 	*errptr=rc;
 
+#if 0
+
+	/*
+	** gnupg returns non-zero exit even if succesfully unencrypted, when
+	** just the signature is bad.
+	*/
 	if (rc == 0)
+#endif
 	{
 		if (fseek(fpout, 0L, SEEK_SET) < 0)
 		{
