@@ -1,9 +1,9 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2006 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: pcpdir.c,v 1.7 2004/05/02 03:25:53 mrsam Exp $";
+static const char rcsid[]="$Id: pcpdir.c,v 1.8 2006/01/23 23:22:10 mrsam Exp $";
 
 #include "config.h"
 #include <stdio.h>
@@ -1294,7 +1294,7 @@ static int docommitevent(struct PCPdir *pd, struct PCPdir_new_eventid *ae,
 									rc= -1;
 								else if (strcmp(delid, ae->eventid.eventid))
 								{
-									int flag;
+									int flag=0;
 									/*
 									** Possibly, but make sure the locking
 									** event actually exists.
@@ -2010,7 +2010,7 @@ static int uncancelevent(struct PCPdir *pd, const char *event, int flags,
 	int found=0;
 	struct PCP_event_time *old_times=NULL;
 	const struct PCP_event_time **old_times_sorted=NULL;
-	unsigned n_old_times;
+	unsigned n_old_times=0;
 
 	if (info) info->errcode=0;
 	dotlock=acquire_dotlock(pd);
