@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2003 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -52,9 +52,10 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.38 2004/07/24 03:36:53 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.42 2006/01/23 23:22:10 mrsam Exp $";
 
 static const char *accessarg=0;
+static const char *accesslocal=0;
 static const char *denymsgarg=0;
 static const char *listenarg=0;
 static const char *ipaddrarg=0;
@@ -80,6 +81,7 @@ static void setup_block(const char *);
 
 static struct args arginfo[]={
 	{"access", &accessarg},
+	{"accesslocal", &accesslocal},
 	{"denymsg", &denymsgarg},
 	{"address", &ipaddrarg},
 	{"block", 0, setup_block},
@@ -509,23 +511,26 @@ int	i;
 gid_t	gid=0;
 const	char *servname;
 int	forced=0;
-int	lockfd;
+int	lockfd=-1;
  
 	argn=argparse(argc, argv, arginfo);
 
-	if (pidarg == 0)
+	if ((stoparg || restartarg) && pidarg == 0)
 	{
 		fprintf(stderr, "%s: -pid argument is required.\n", argv[0]);
 		return (-1);
 	}
 
-	lockfilename=malloc(strlen(pidarg)+sizeof(".lock"));
-	if (!lockfilename)
+	if (pidarg)
 	{
-		perror("malloc");
-		return (-1);
+		lockfilename=malloc(strlen(pidarg)+sizeof(".lock"));
+		if (!lockfilename)
+		{
+			perror("malloc");
+			return (-1);
+		}
+		strcat(strcpy(lockfilename, pidarg), ".lock");
 	}
-	strcat(strcpy(lockfilename, pidarg), ".lock");
 
 	if (stoparg)
 	{
@@ -546,11 +551,13 @@ int	lockfd;
 		return (-1);
 	}
 
-	lockfd=ll_daemon_start(lockfilename);
-	if (lockfd < 0)
-	{
-		perror("ll_daemon_start");
-		return (-1);
+	if (pidarg) { /* -start implied for backwards compatibility */
+		lockfd=ll_daemon_start(lockfilename);
+		if (lockfd < 0)
+		{
+			perror("ll_daemon_start");
+			return (-1);
+		}
 	}
 
 	servname=argv[argn++];
@@ -607,7 +614,8 @@ int	lockfd;
 	}
 #endif
 
-	ll_daemon_started(pidarg, lockfd);
+	if (pidarg)
+		ll_daemon_started(pidarg, lockfd);
 
 	if (grouparg)
 	{
@@ -642,7 +650,7 @@ int	lockfd;
 		}
 	}
 
-	if (ll_daemon_resetio())
+	if (pidarg && ll_daemon_resetio())
 	{
 		perror("ll_daemon_resetio");
 		close(lockfd);
@@ -815,22 +823,22 @@ static RETSIGTYPE childsig(int signum)
 #endif
 }
 
-static int doallowaccess(char *);
+static int doallowaccess(char *, int);
 
 #if	RFC1035_IPV6
 
-static int allowaccess(const RFC1035_ADDR *sin)
+static int allowaccess(const RFC1035_ADDR *sin, int port)
 {
-char	buf[RFC1035_MAXNAMESIZE+1];
+char	buf[RFC1035_MAXNAMESIZE+1+6];
 char	*q;
 int	i;
 
 	if (IN6_IS_ADDR_V4MAPPED(sin))
 	{
-	const char *p=inet_ntop(AF_INET6, sin, buf, sizeof(buf));
+	const char *p=inet_ntop(AF_INET6, sin, buf, sizeof(buf)-6);
 
 		if (p && (q=strrchr(buf, ':')) != 0)
-			return (doallowaccess(q+1));
+			return (doallowaccess(q+1, port));
 		return (1);
 	}
 	q=buf;
@@ -843,20 +851,20 @@ int	i;
 		q += 5;
 	}
 	*q=0;
-	return (doallowaccess(buf));
+	return (doallowaccess(buf, port));
 }
 
 #else
-static int allowaccess(const RFC1035_ADDR *sin)
+static int allowaccess(const RFC1035_ADDR *sin, int port)
 {
-char	buf[RFC1035_NTOABUFSIZE];
+char	buf[RFC1035_NTOABUFSIZE+6];
 
 	rfc1035_ntoa(sin, buf);
-	return (doallowaccess(buf));
+	return (doallowaccess(buf, port));
 }
 #endif
 
-static int doallowaccess(char *buf)
+static int doallowaccess(char *buf, int port)
 {
 char	*accessptr;
 char *p, *q, *r;
@@ -865,6 +873,7 @@ int	l;
 
 	if (accessarg == 0)	return (1);
 
+	if (port) snprintf(buf+strlen(buf), 7, ".%d", ntohs(port));
 	while ((accessptr= *buf ? chkaccess(buf):0) == 0)
 	{
 		if ((accessptr=strrchr(buf, '.')) == 0
@@ -873,6 +882,12 @@ int	l;
 #endif
 			)
 		{
+			if (port)
+			{
+				snprintf(buf, 8, "*.%d", ntohs(port));
+				if ((accessptr=chkaccess(buf)) != 0)
+					break;
+			}
 			if ((accessptr=chkaccess("*")) != 0)
 				break;
 			return (1);
@@ -1041,7 +1056,7 @@ static int doit(int argn, int argc, char **argv)
 		int n;
 		int sockfd;
 		RFC1035_NETADDR	sin;
-		int	sinl;
+		socklen_t	sinl;
 
 		fdr=fdrcopy;
 
@@ -1157,7 +1172,24 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 	{
 		wait_restore(childsig);
 
-		if (allowaccess(&addr) == 0)
+		if (accesslocal) // Lookup local interface address too?
+		{
+		RFC1035_NETADDR lsin;
+		RFC1035_ADDR laddr;
+		int	lport;
+		socklen_t	i=sizeof(lsin);
+		
+			if (sox_getsockname(sockfd, (struct sockaddr *)&lsin, &i) == 0 &&
+				rfc1035_sockaddrip(&lsin, i, &laddr) == 0 &&
+				rfc1035_sockaddrport(&lsin, i, &lport) == 0 &&
+				allowaccess(&laddr,lport) == 0)
+			{
+				sox_close(sockfd);
+				_exit(0);
+			}
+		}
+
+		if (allowaccess(&addr,0) == 0)
 		{
 			if (denymsgarg) {
 				write(sockfd, denymsgarg, strlen(denymsgarg));
@@ -1396,7 +1428,7 @@ int i;
 					RFC1035_RESOLVE_RECURSIVE,
 					hostname,
 					RFC1035_TYPE_TXT,
-					RFC1035_CLASS_IN, &replyp)) >= 0)
+					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
 			{
 				rfc1035_rr_gettxt(replyp->allrrs[i], 0,
 					hostname);
@@ -1416,7 +1448,7 @@ int i;
 					RFC1035_RESOLVE_RECURSIVE,
 					hostname,
 					RFC1035_TYPE_A,
-					RFC1035_CLASS_IN, &replyp)) >= 0)
+					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
 			{
 				mkmymsg(varname, p->msg);
 			}
@@ -1450,7 +1482,7 @@ int i;
 			RFC1035_RESOLVE_RECURSIVE,
 			hostname,
 			hasnotxt ? RFC1035_TYPE_ANY:RFC1035_TYPE_A,
-			RFC1035_CLASS_IN, &replyp);
+			RFC1035_CLASS_IN, &replyp, 0);
 
 	if (!replyp)	return;
 
@@ -1556,7 +1588,7 @@ RFC1035_NETADDR lsin;
 RFC1035_ADDR laddr;
 int	lport;
 
-int	i;
+socklen_t	i;
 int	ipcnt, ccnt;
 char	buf[RFC1035_MAXNAMESIZE+128];
 struct blocklist_s *bl;
@@ -1652,6 +1684,8 @@ int rc;
 	}
 	if (accessarg && openaccess(accessarg))
 		perror(accessarg);
+	if (accesslocal && !accessarg)
+		fprintf(stderr,"-accesslocal requires -access\n");
 	rc=doit(argn, argc, argv);
 	kill( -getpid(), SIGTERM);
 	exit(rc);
