@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2005 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: attachments.c,v 1.41 2004/05/23 14:28:24 mrsam Exp $
+** $Id: attachments.c,v 1.46 2006/05/03 01:57:06 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -21,6 +21,7 @@
 #include	"newmsg.h"
 #include	"gpg.h"
 #include	"gpglib/gpglib.h"
+#include	"courierauth.h"
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<ctype.h>
@@ -50,6 +51,7 @@
 #endif
 
 extern char *newmsg_alladdrs(FILE *);
+extern void newmsg_copy_content_headers(FILE *fp);
 
 extern char *alloc_filename(const char *, const char *, const char *);
 extern const char *showsize(unsigned long);
@@ -72,7 +74,23 @@ extern const char *sqwebmail_content_language;
 
 static void attachment_showname(const char *);
 
-static off_t max_attach = MAXMSGSIZE;
+#define HASTEXTPLAIN(q) (rfc2045_searchcontenttype((q), "text/plain") != NULL)
+/* Also in newmsg_create.c */
+
+
+static off_t max_attach()
+{
+	off_t n=0;
+	const char *p=getenv("SQWEBMAIL_MAXMSGSIZE");
+
+	if (p)
+		n=atol(p);
+
+	if (n < MAXMSGSIZE)
+		n=MAXMSGSIZE;
+	return n;
+}
+
 
 void attachments_head(const char *folder, const char *pos, const char *draft)
 {
@@ -122,7 +140,7 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 
 	if (strcmp(cgi("error"), "limits") == 0)
 	{
-		printf(limiterr, (unsigned long)(max_attach / 1024));
+		printf(limiterr, (unsigned long)(max_attach() / (1024 * 1024)));
 	}
 
 	if (strcmp(cgi("error"), "makemime") == 0)
@@ -134,6 +152,22 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 	tokennew();
 	printf("<table width=\"100%%\" border=\"0\">");
 
+	if (rfcp)
+	{
+		const char *content_type;
+		const char *content_transfer_encoding;
+		const char *charset;
+
+		rfc2045_mimeinfo(rfcp, &content_type,
+			&content_transfer_encoding, &charset);
+
+		if (content_type &&
+		    strcmp(content_type, "multipart/alternative") == 0)
+			rfcp=NULL;
+
+		/* No attachments here */
+	}
+
 	for (q=rfcp ? rfcp->firstpart:0; q; q=q->next)
 	{
 	const char *content_type;
@@ -149,8 +183,7 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 
 		rfc2045_mimeinfo(q, &content_type,
 			&content_transfer_encoding, &charset);
-		if (!foundtextplain && strcmp(content_type,
-				"text/plain") == 0)
+		if (!foundtextplain && HASTEXTPLAIN(q))
 		{
 			foundtextplain=1;
 			continue;
@@ -228,7 +261,7 @@ void attachments_opts(const char *draft)
 	printf("<label><input type=\"checkbox\" name=\"fcc\"%s />%s</label><br />",
 	       pref_noarchive ? "":" checked=\"checked\"",
 	       getarg("PRESERVELAB"));
-	if (access(NODSN, R_OK))
+	if (auth_getoptionenvint("wbnodsn") == 0)
 		printf("<label><input type=\"checkbox\" name=\"dsn\" />%s</label><br />",
 		       getarg("DSN"));
 
@@ -333,8 +366,7 @@ char	buf[MAXLONGSIZE+4];
 			&content_transfer_encoding, &charset);
 		if (q->isdummy)	continue;
 
-		if (strcmp(content_type, "text/plain") == 0 &&
-			!foundtextplain)
+		if (!foundtextplain && HASTEXTPLAIN(q))
 		{
 			foundtextplain=1;
 			continue;
@@ -360,21 +392,17 @@ off_t dummy;
 		if (q->isdummy)	continue;
 		rfc2045_mimeinfo(q, &content_type,
 			&content_transfer_encoding, &charset);
-		if (strcmp(content_type, "text/plain") == 0)
+		if (HASTEXTPLAIN(q))
 			break;
 	}
 	if (!q)	return (-1);
 
 	if (fseek(fp, 0L, SEEK_SET) == -1)	return (-1);
 	newmsg_copy_nonmime_headers(fp);
-	maildir_writemsgstr(newdraftfd, "mime-version: 1.0\ncontent-type: text/plain; charset=\"");
-	maildir_writemsgstr(newdraftfd, sqwebmail_content_charset);
-	maildir_writemsgstr(newdraftfd, "\"\ncontent-transfer-encoding: ");
-	maildir_writemsgstr(newdraftfd, content_transfer_encoding);
-	maildir_writemsgstr(newdraftfd, "\n\n");
+	maildir_writemsgstr(newdraftfd, "mime-version: 1.0\n");
 
 	rfc2045_mimepos(q, &start_pos, &end_pos, &start_body, &dummy, &dummy);
-	return (messagecopy(fp, start_body, end_pos));
+	return (messagecopy(fp, start_pos, end_pos));
 }
 
 static int del_some_attachments(FILE *fp, struct rfc2045 *rfcp)
@@ -401,8 +429,7 @@ off_t	dummy;
 			&content_transfer_encoding, &charset);
 		if (q->isdummy)
 			;
-		else if (strcmp(content_type, "text/plain") == 0 &&
-			!foundtextplain)
+		else if (!foundtextplain && HASTEXTPLAIN(q))
 			foundtextplain=1;
 		else
 		{
@@ -647,7 +674,7 @@ int attach_upload(const char *draft,
 	FILE	*draftfp;
 	char	*boundary;
 	FILE	*tempfp;
-	struct	rfc2045 *rfcp;
+	struct	rfc2045 *rfcp, *q;
 	const char *content_type;
 	const char *content_transfer_encoding;
 	const char *charset;
@@ -714,7 +741,7 @@ int attach_upload(const char *draft,
 	}
 
 	if (fstat(attachfd, &attach_stat_buf) ||
-		attach_stat_buf.st_size + stat_buf.st_size > max_attach)
+	    attach_stat_buf.st_size + stat_buf.st_size > max_attach())
 	{
 		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		maildir_deletenewmsg(attachfd, INBOX "." DRAFTS, attachfilename);
@@ -791,22 +818,28 @@ int attach_upload(const char *draft,
 	maildir_writemsgstr(newdraftfd, boundary);
 	maildir_writemsgstr(newdraftfd, "\n");
 
-	if (!rfcp->firstpart)
+	if (rfcp == NULL || strcmp(content_type, "multipart/mixed"))
 	{
-		/* The current draft does not have attachments.  Take its
+		int rc;
+
+		/*
+		** The current draft does not have attachments.  Take its
 		** sole contents, and write it as a text/plain attachment.
 		*/
 
-		maildir_writemsgstr(newdraftfd, "Content-Type: ");
-		maildir_writemsgstr(newdraftfd, content_type);
-		maildir_writemsgstr(newdraftfd, "; format=xdraft; charset=\"");
-		maildir_writemsgstr(newdraftfd, sqwebmail_content_charset);
-		maildir_writemsgstr(newdraftfd, "\"\nContent-Transfer-Encoding: ");
-		maildir_writemsgstr(newdraftfd, content_transfer_encoding);
-		maildir_writemsgstr(newdraftfd, "\n\n");
-		rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
-			&dummy, &dummy);
-		if (messagecopy(draftfp, start_body, end_pos))
+		if (fseek(draftfp, 0L, SEEK_SET) < 0)
+			rc = -1;
+		else
+		{
+			newmsg_copy_content_headers(draftfp);
+			maildir_writemsgstr(newdraftfd, "\n");
+			rfc2045_mimepos(rfcp, &start_pos, &end_pos,
+					&start_body,
+					&dummy, &dummy);
+			rc=messagecopy(draftfp, start_body, end_pos);
+		}
+
+		if (rc)
 		{
 			maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename,
 				0, 0);
@@ -824,8 +857,6 @@ int attach_upload(const char *draft,
 	}
 	else
 	{
-	struct rfc2045 *q;
-
 		/* If the current draft already has MIME attachments,
 		** just copy them over to the new draft message.
 		*/
