@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: auth.c,v 1.37 2004/05/09 03:32:01 mrsam Exp $
+** $Id: auth.c,v 1.40 2004/11/21 03:38:35 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
@@ -25,21 +25,18 @@
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<ctype.h>
+#include	<courierauth.h>
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
 #include	<string.h>
 #include	<errno.h>
 
-#include	"authlib/auth.h"
-#include	"authlib/authmod.h"
-#include	"authlib/authstaticlist.h"
 #include	"maildir/loginexec.h"
-
+#include	"courierauthdebug.h"
 #include	"auth.h"
 #include	"htmllibdir.h"
 
-
 extern int check_sqwebpass(const char *);
 
 #if	HAVE_SQWEBMAIL_UNICODE
@@ -88,198 +85,93 @@ static int login_maildir(const char *maildir)
 	return (0);
 }
 
-#if 0
-struct preinfo {
-	int i;
-	int haswebpass;
-	} ;
-
-static int pre_callback(struct authinfo *authinfo, void *vp)
-{
-const char *h=authinfo->homedir;
-struct preinfo *pi= (struct preinfo *)vp;
-
-	if (!h)	return (-1);
-
-	pi->i=authinfo->staticindex;
-	pi->haswebpass=0;
-
-#if ENABLE_WEBPASS
-   {
-const char *p=authinfo->maildir;
-char	*buf;
-struct stat stat_buf;
-
-	if (!p || !*p)	p=getenv("MAILDIRPATH");
-	if (!p || !*p)	p="Maildir";
-
-	buf=malloc(strlen(h)+strlen(p)+sizeof(PASSFILE)+100);
-
-	if (!buf)
-	{
-		perror("malloc");
-		return (1);
-	}
-	if (*p == '/')
-		strcpy(buf, p);
-	else
-		strcat(strcat(strcpy(buf, h), "/"), p);
-	strcat(buf, "/" PASSFILE);
-
-	if (stat(buf, &stat_buf) == 0)	pi->haswebpass=1;
-	free(buf);
-   }
-#endif
-
-	return (0);
-}
-#endif
-
 static int doauthlogin(struct authinfo *a, void *vp)
 {
-        authsuccess(a->homedir, a->sysuserid ? 0:a->sysusername,
-			a->sysuserid, &a->sysgroupid,
-			a->address, a->fullname);
+	const char *p=auth_getoption(a->options ? a->options:"",
+				     "disablewebmail");
+	const char *c=(const char *)vp;
+
+	static char *authaddr=NULL;
+	static char *authfullname=NULL;
+	static char *authoptions=NULL;
+	const char *n;
+	char *b;
+	int rc;
+
+
+	if (p && atoi(p))
+		return -1;
+
+	if ((rc = auth_callback_default(a)) != 0)
+	{
+		if (rc > 0)
+			perror("ERR: authentication error");
+		return -1;
+	}
+
 	if (login_maildir(a->maildir))
+	{
+		error("Unable to open the maildir for this account -- the maildir doesn't exist or has incorrect ownership or permissions.");
 		return (-1);
-	return (0);
-}
-
-static const char *do_login(const char *u, const char *p, const char **driver,
-			    int *has_changepwd)
-{
-char	*buf;
-static char	*uid=0;
-int	i;
-
-#if 0
-struct	preinfo pi;
-int	rc;
-
-/*
-** We first need to check if sqwebmail saved its own password in sqwebmail-pass.
-** To do that, we call the authlib preauthentication functions until we find
-** one that accepts our userid.  However, if it succeeds, we will no longer
-** as root.  Therefore, let's fork, and do our thing in the child proc.
-*/
-
-	pi.haswebpass=0;
-	pi.i=0;
-
-	rc=authstaticlist_search_string(u, "webmail", getenv("AUTHMODULES"),
-		&pre_callback, &pi);
-
-	if (rc)
-		return (0);
-
-#if ENABLE_WEBPASS
-	if (pi.haswebpass)
-	{
-		if ((*authstaticmodulelist[pi.i]->auth_prefunc)(u, "webmail",
-                        &doauthlogin, 0))
-		{
-			return (0);
-		}
-
-		if (check_sqwebpass(p) == 0)
-		{
-			*driver=authstaticmodulelist[pi.i]->auth_name;
-			*has_changepwd=
-				authstaticmodulelist[pi.i]->auth_changepwd
-				!= 0;
-			return (u);
-		}
-		return (0);
 	}
-#endif
 
-#endif
+	b=malloc(sizeof("AUTHADDR=")+strlen(c));
 
-	/* Let the authlib library check the password instead, then we'll
-	** adopt it as our own.
-	*/
-
-	if ((buf=malloc(strlen(u)+strlen(p)+3)) == 0)
+	if (!b)
 		enomem();
-
-	strcat(strcat(strcat(strcpy(buf, u), "\n"), p), "\n");
-
-	if (uid)	free(uid);
-
-	uid=authlogin_search_string(getenv("AUTHMODULES"), "webmail",
-				    AUTHTYPE_LOGIN, buf, 0, NULL, NULL, &i);
-	free(buf);
-
-	if (uid != 0)
-	{
-		char *p=authgetoptionenv("disablewebmail");
-
-		if (p && atoi(p))
-		{
-			free(p);
-			return NULL;
-		}
-		free(p);
-
-		if (login_maildir(getenv("MAILDIR")))
-			error("Unable to open the maildir for this account -- the maildir doesn't exist or has incorrect ownership or permissions.");
-
-		*driver=authstaticmodulelist[i]->auth_name;
-		*has_changepwd=
-			authstaticmodulelist[i]->auth_changepwd
-			!= 0;
-		return (uid);
-	}
-	return (0);
-}
+	strcat(strcpy(b, "AUTHADDR="), c);
+	putenv(b);
+	if (authaddr)
+		free(authaddr);
+	authaddr=b;
 
 
-static int badstr(const char *p)
-{
-	while (*p)
-	{
-		if ((int)(unsigned char)*p < ' '
-		    || *p == '\\' || *p == '\'' || *p == '"')
-			return (1);
-		++p;
-	}
-	return (0);
-}
+	n=a->fullname;
+	if (!n) n="";
+	b=malloc(sizeof("AUTHFULLNAME=")+strlen(n));
 
-static void verifyuid(char *uid)
-{
-	if (badstr(uid))
-	{
-		free(uid);
+	if (!b)
 		enomem();
-	}
+	strcat(strcpy(b, "AUTHFULLNAME="), n);
+	putenv(b);
+	if (authfullname)
+		free(authfullname);
+	authfullname=b;
+
+	n=a->options;
+	if (!n) n="";
+
+	b=malloc(sizeof("OPTIONS=")+strlen(n));
+
+	if (!b)
+		enomem();
+	strcat(strcpy(b, "OPTIONS="), n);
+	putenv(b);
+	if (authoptions)
+		free(authoptions);
+	authoptions=b;
+
+	return (0);
 }
 
-/*
-** The returned mailboxid is of the form user.authdriver
-**
-** authdriver is included so that we can find the same authentication module
-** quickly, for each request.  authdriver is appended, rather than prepended,
-** because the logincache hashes on the first couple of characters of the id.
-*/
-
-const char *login(const char *uid, const char *pass, int *can_changepwd)
+const char *do_login(const char *u, const char *p, const char *ip)
 {
-const char *driver;
-const char *p=do_login(uid, pass, &driver, can_changepwd);
-static char *login_id_buf=0;
-
-	if (p)
+	if (auth_login("webmail", u, p, doauthlogin, (void *)u))
 	{
-		if (badstr(uid) || badstr(pass))
-			return (NULL);
-
-		if (login_id_buf)	free(login_id_buf);
-		if ((login_id_buf=malloc(strlen(p)+strlen(driver)+2)) == 0)
-			enomem();
-		p=strcat(strcat(strcpy(login_id_buf, p), "."), driver);
+		courier_safe_printf("INFO: LOGIN FAILED, user=%s, ip=[%s]",
+				  u?u:"", ip);
+		return NULL;
 	}
-	return (p);
+
+	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n", u, ip);
+	return u;
+}
+
+int nochangepass()
+{
+	if (auth_getoptionenvint("wbnochangepass")) return 1;
+	if (access(SQWEBPASSWD, X_OK)) return 1;
+	return 0;
 }
 
 /*
@@ -297,81 +189,29 @@ static char *login_id_buf=0;
 int login_changepwd(const char *u, const char *oldpwd, const char *newpwd,
 		    int *rc)
 {
-	char *uid=strdup(u);
-	char *driver;
-	int	i;
+	if (nochangepass())
+		return -1;
 
-	if (!uid)
-		enomem();
+	*rc= -1;
 
-	verifyuid(uid);
-
-	if ((driver=strrchr(uid, '.')) == 0)
+	if (changepw("webmail", u, oldpwd, newpwd) == 0)
 	{
-		free(uid);
-		enomem();
+		*rc=0;
 	}
-
-	*driver++=0;
-
-	for (i=0; authstaticmodulelist[i]; i++)
-	{
-		if (strcmp(authstaticmodulelist[i]->auth_name, driver) == 0)
-		{
-			*rc=badstr(uid) || badstr(oldpwd) || badstr(newpwd)
-				? 1:(*authstaticmodulelist[i]->auth_changepwd)
-				("webmail", uid, oldpwd, newpwd);
-
-			free(uid);
-			return (0);
-		}
-	}
-	return (-1);
+	return 0; // TODO
 }
 
 int prelogin(const char *u)
 {
-char	*p=malloc(strlen(u)+1);
-char	*driver;
-int	i;
-
-	if (!p)	enomem();
-	strcpy(p, u);
-
-	verifyuid(p);
-
-	if ((driver=strrchr(p, '.')) == 0)
-	{
-		free(p);
-		enomem();
-	}
-	*driver++ = 0;
-
-	for (i=0; authstaticmodulelist[i]; i++)
-	{
-		if (strcmp(authstaticmodulelist[i]->auth_name, driver) == 0)
-		{
-			if ((*authstaticmodulelist[i]->auth_prefunc)(p,
-								     "webmail",
-				&doauthlogin, 0) == 0)
-			{
-				free(p);
-				return (0);
-			}
-			free(p);
-			return (-1);
-		}
-	}
-	free(p);
-	return (-1);
+	return auth_getuserinfo("webmail", u, doauthlogin, (void *)u);
 }
 
 const char *login_returnaddr()
 {
-const char *p=getenv("AUTHADDR");
-const char *domain=myhostname();
+	const char *p=getenv("AUTHADDR");
+	const char *domain=myhostname();
 
-static char *addrbuf=0;
+	static char *addrbuf=0;
 
 	if (!p)	p="";
 	if (addrbuf)	free(addrbuf);
