@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: ispell.c,v 1.3 2002/01/08 05:18:00 mrsam Exp $
+** $Id: ispell.c,v 1.4 2004/11/04 03:01:00 mrsam Exp $
 */
 #include	"ispell.h"
 #include	<stdio.h>
@@ -59,15 +59,23 @@ int	argn;
 	prevsighandler=signal(SIGCHLD, SIG_DFL);
 	signal(SIGCHLD, prevsighandler);
 
-	if (pipe(fdbuf) < 0)	return (-1);
+	if (pipe(fdbuf) < 0)
+	{
+		perror("pipe");
+		return (-1);
+	}
 
 	switch (ispell_pid=fork())	{
 	case -1:
+		perror("fork");
 		close(fdbuf[0]);
 		close(fdbuf[1]);
 		return (-1);
 	case 0:
-		if (pipe(fdbuf2) < 0)	_exit(-1);
+		if (pipe(fdbuf2) < 0) {
+			perror("pipe");
+			_exit(-1);
+		}
 
 		/*
 		** First child will fork and run the real ispell, and write to
@@ -76,6 +84,7 @@ int	argn;
 
 		switch (ispell_pid=fork())	{
 		case -1:
+			perror("fork");
 			_exit(-1);
 		case 0:
 			close(0);
@@ -96,6 +105,7 @@ int	argn;
 			}
 			args[argn]=0;
 			execv(ISPELL, (char **)args);
+			perror("fork_ispell: execv " ISPELL);
 			_exit(1);
 		}
 		close(fdbuf[0]);
@@ -107,11 +117,19 @@ int	argn;
 		{
 		int	n=write(fdbuf2[1], ispellline, l);
 
-			if (n <= 0)	_exit(1);
+			if (n <= 0)
+			{
+				perror("fork_ispell: write");
+				_exit(1);
+			}
 			ispellline += n;
 			l -= n;
 		}
-		if (write(fdbuf2[1], "\n", 1) != 1)	_exit(1);
+		if (write(fdbuf2[1], "\n", 1) != 1)
+		{
+			perror("fork_ispell: write");
+			_exit(1);
+		}
 		close(fdbuf2[1]);
 		while ( wait(&waitstat) != ispell_pid)
 			;
@@ -122,6 +140,7 @@ int	argn;
 	ispell_resp=fdopen(fdbuf[0], "r");
 	if (!ispell_resp)
 	{
+		perror("fork_ispell: fdopen");
 		close(fdbuf[0]);
 		wait4pid();
 		return (-1);
@@ -139,6 +158,7 @@ int	argn;
 					:malloc(l);
 			if (!newbuf)
 			{
+				perror("fork_ispell: malloc/realloc");
 				if (isp->ispell_buf)	free(isp->ispell_buf);
 				isp->ispell_buf=0;
 				wait4pid();
@@ -171,7 +191,10 @@ int	nmisses;
 struct	ispell_suggestion **sp;
 
 	if ((isp=(struct ispell *)malloc(sizeof(struct ispell))) == 0)
+	{
+		perror("malloc");
 		return (0);
+	}
 	isp->ispell_buf=0;
 	isp->first_misspelled=0;
 	islistptr=&isp->first_misspelled;
@@ -179,11 +202,13 @@ struct	ispell_suggestion **sp;
 	if (fork_ispell(line, dict, isp))
 	{
 		ispell_free(isp);
+		fprintf(stderr, "ERR: ispell_run: fork_ispell failed\n");
 		return (0);
 	}
 
 	if ((p=strchr(isp->ispell_buf, '\n')) == 0)
 	{
+		fprintf(stderr, "ERR: ispell_run: incomplete result from ispell\n");
 		ispell_free(isp);
 		return (0);
 	}
@@ -200,6 +225,7 @@ struct	ispell_suggestion **sp;
 		if ( (ip=*islistptr=(struct ispell_misspelled *)
 			malloc(sizeof(struct ispell_misspelled))) == 0)
 		{
+			perror("malloc");
 			ispell_free(isp);
 			return (0);
 		}
@@ -235,6 +261,7 @@ struct	ispell_suggestion **sp;
 				malloc(sizeof(struct ispell_suggestion)))
 				== 0)
 			{
+				perror("malloc");
 				ispell_free(isp);
 				return (0);
 			}
