@@ -1,5 +1,5 @@
 /*
-** $Id: mailfilter.c,v 1.16 2004/02/17 00:58:24 mrsam Exp $
+** $Id: mailfilter.c,v 1.18 2006/05/13 02:54:23 mrsam Exp $
 */
 
 /*
@@ -177,6 +177,8 @@ struct maildirfilterrule *r;
 					"nothasmultipart":
 					"hasmultipart":
 			r->type == islargerthan ? "hassize":
+			r->type == anymessage
+					? "anymessage":
 			r->type == textplain ?
 				r->flags & MFR_DOESNOT ?
 					"nothastextplain":
@@ -309,6 +311,10 @@ struct maildirfilterrule *r;
 				enomem();
 			cgi_put("autoresponse_from", fromhdr);
 		}
+		else if (strcmp(actionbuf, "exit") == 0)
+		{
+			cgi_put("action", "purge");
+		}
 		else
 		{
 			cgi_put("action", "savefolder");
@@ -454,6 +460,10 @@ const char *autoreply_from="";
 		}
 		fieldvalue=libmail_str_size_t(n, numbuf);
 	}
+	else if (strcmp(p, "anymessage") == 0)
+	{
+		type=anymessage;
+	}
 	else
 	{
 		if (strcmp(p, "body") == 0)
@@ -534,6 +544,11 @@ const char *autoreply_from="";
 		free(q);
 		autoreply_from=cgi("autoresponse_from");
 	}
+	else if (strcmp(p, "purge") == 0)
+	{
+		tofolder = strdup("exit");
+		if (!tofolder) enomem();
+	}
 	else
 	{
 		tofolder=strdup(cgi("savefolder"));
