@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2005 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: newmsg.c,v 1.54 2004/05/23 14:28:24 mrsam Exp $
+** $Id: newmsg.c,v 1.57 2005/11/23 02:37:23 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -27,6 +27,7 @@
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045/rfc2045.h"
 #include	"gpglib/gpglib.h"
+#include	"courierauth.h"
 #include	<string.h>
 #include	<stdio.h>
 #include	<signal.h>
@@ -253,9 +254,45 @@ int	fd;
 }
 #endif
 
+
+/*
+** Strip leading space from flowed text format-encoded message.
+*/
+
+static int textarea_eol;
+
 static int show_textarea(const char *p, size_t l, void *voidptr)
 {
-	print_attrencodedlen(p, l, 1, stdout);
+	const char *ptr=p;
+	size_t cnt=0;
+
+	while (l)
+	{
+		if (textarea_eol)
+		{
+			textarea_eol=0;
+			if (*p == ' ')
+			{
+				if (cnt)
+					print_attrencodedlen(ptr, cnt,
+							     1, stdout);
+				++p;
+				--l;
+				ptr=p;
+				cnt=0;
+				continue;
+			}
+		}
+
+		if (*p == '\n')
+			textarea_eol=1;
+		++p;
+		--l;
+		++cnt;
+	}
+
+	if (cnt)
+		print_attrencodedlen(ptr, cnt, 1, stdout);
 	return (0);
 }
 
@@ -365,8 +402,22 @@ void newmsg_showfp(FILE *fp, int *attachcnt)
 	if (*attachcnt)	--*attachcnt;
 	/* Not counting the 1st MIME part */
 
+	{
+		const char *content_type;
+		const char *content_transfer_encoding;
+		const char *charset;
+
+		rfc2045_mimeinfo(p, &content_type,
+				 &content_transfer_encoding, &charset);
+
+		if (content_type &&
+		    strcmp(content_type, "multipart/alternative") == 0)
+			*attachcnt=0;
+	}
+
 	q=rfc2045_searchcontenttype(p, "text/plain");
 
+	textarea_eol=1;
 	if (q)
 #if	HAVE_SQWEBMAIL_UNICODE
 		rfc2045_decodetextmimesection(fileno(fp), q,
@@ -569,7 +620,7 @@ void newmsg_init(const char *folder, const char *pos)
 	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"4\" class=\"new-message-box\"><tr><td>\n");
 
 	printf("<table border=\"0\" width=\"100%%\">\n");
-	if (access(NOCHANGINGFROM, 0))
+	if (auth_getoptionenvint("wbnochangingfrom") == 0)
 		newmsg_header_rfc822(fromlab, "headerfrom", curfrom,
 			*cgi("from") ? cgi("from"):
 			pref_from && *pref_from ? pref_from:
@@ -611,10 +662,28 @@ void newmsg_init(const char *folder, const char *pos)
 	if (cursubj)	free(cursubj);
 
 	printf("<tr><td colspan=\"3\"><hr width=\"100%%\" /></td></tr>");
-	printf("<tr><th valign=\"top\" align=\"right\">"
+	printf("<tr>"
+
+	       "<th valign=\"top\" align=\"right\">"
 	       "<p class=\"new-message-header\">"
 	       "<span class=\"new-message-header-message\">"
-	       "%s</span></p></th><td width=\"6\">&nbsp;</td>", messagelab);
+	       "%s</span></p></th><td width=\"6\">&nbsp;</td>"
+	       "<td><select name=\"textformat\"><option value=\"plain\" %s>%s"
+	       "<option value=\"wiki\" %s>%s</select>"
+
+	       "<small>(<a href='%s/wikifmt.html' target='_blank'>%s</a>)</small>"
+	       "</td></tr>"
+	       "<th valign=\"top\" align=\"right\">"
+	       "<p class=\"new-message-header\">"
+	       "<span class=\"new-message-header-message\">"
+	       "%s</span></p></th><td width=\"6\">&nbsp;</td>",
+
+	       getarg("FMTNAME"),
+	       pref_wikifmt ? "":"selected='selected'", getarg("FMTTEXTPLAIN"),
+	       pref_wikifmt ? "selected='selected'":"", getarg("FMTTEXTWIKI"),
+
+	       get_imageurl(), getarg("FMTHELP"),
+	       messagelab);
 
 	printf("<td><textarea name=\"message\" cols=\"%d\" rows=\"15\" wrap=\"soft\">",
 		MYLINESIZE);
@@ -647,6 +716,7 @@ void newmsg_init(const char *folder, const char *pos)
 	{
 		printf("<input type=\"hidden\" name=\"draftmessage\" value=\"");
 		output_attrencoded(draftmessage);
+
 		printf("\" />");
 	}
 	if (draftmessage)	free(draftmessage);
@@ -662,7 +732,7 @@ void newmsg_init(const char *folder, const char *pos)
 	       pref_noarchive ? "":" checked=\"checked\"",
 	       getarg("PRESERVELAB"));
 
-	if (access(NODSN, R_OK))
+	if (auth_getoptionenvint("wbnodsn") == 0)
 		printf("<tr><td colspan=\"2\" align=\"right\"><input type=\"checkbox\" "
 		       "name=\"dsn\" id=\"dsn\" /></td><td><label for=\"dsn\">%s</label></td></tr>\n",
 		       getarg("DSN"));
