@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: pcp.c,v 1.26 2004/05/02 03:25:53 mrsam Exp $
+** $Id: pcp.c,v 1.29 2006/03/12 13:56:53 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"pcp.h"
@@ -27,6 +27,7 @@
 #include	"maildir.h"
 #include	"newmsg.h"
 #include	"pref.h"
+#include	"courierauth.h"
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
@@ -82,6 +83,9 @@ extern int attach_upload(const char *,
 extern void newmsg_showfp(FILE *, int *);
 
 static struct PCP *calendar=NULL;
+static void refreshcache(struct PCP *);
+extern size_t get_timeoutsoft();
+
 
 /*
 ** CGI process startup
@@ -141,14 +145,14 @@ static const char *sqpcp_mode()
 ** Log in to calendar.
 */
 
-static void do_login(const char *, const char *, int);
+static void do_pcplogin(const char *userid, const char *password, int showerr);
 
 void sqpcp_login(const char *userid, const char *password)
 {
-	do_login(userid, password, 0);
+	do_pcplogin(userid, password, 0);
 }
 
-static void do_login(const char *userid, const char *password, int showerr)
+static void do_pcplogin(const char *userid, const char *password, int showerr)
 {
 	struct PCP *pcp;
 
@@ -199,6 +203,7 @@ static void do_login(const char *userid, const char *password, int showerr)
 			}
 		}
 		pcp_cleanup(pcp);
+		refreshcache(pcp);
 	}
 }
 
@@ -238,8 +243,6 @@ static void destroycache(struct cacherecord *, unsigned);
 
 static int need_refresh();
 
-static void refreshcache(struct PCP *);
-
 int sqpcp_loggedin()
 {
 	struct PCP *pcp;
@@ -265,33 +268,58 @@ int sqpcp_loggedin()
 
 /* Check if it's time to rebuild CACHE file */
 
+static const char *parsetimet(const char *buf, time_t *tp)
+{
+	*tp=libmail_strtotime_t(&buf);
+
+	if (*buf == '\t')
+		++buf;
+	return buf;
+}
+
 static int need_refresh()
 {
 	FILE *fp;
 	struct stat stat_buf;
 
-	if (stat(CHANGEDFILE, &stat_buf) == 0)
-		return (1);
-
 	/* Check whether it's time to rebuild the cache */
 
 	fp=fopen(CACHE, "r");
 
+	if (stat(CHANGEDFILE, &stat_buf) == 0)
+	{
+		if (fp)
+			fclose(fp);
+		return (1);
+	}
+
 	if (fp)
 	{
 		struct stat stat_buf;
 
 		char buffer[BUFSIZ];
-		unsigned long a, b;
+		const char *p;
 
+		time_t a, b;
+
+		int ch=getc(fp);
+
+		if (ch == EOF)
+		{
+			fclose(fp);
+			return (0); /* No events scheduled */
+		}
+
+		ungetc(ch, fp);
 		if (fgets(buffer, sizeof(buffer), fp) &&
-		    sscanf(buffer, "%lu\t%lu", &a, &b) == 2)
+		    (p=parsetimet(buffer, &a)) &&
+		    parsetimet(p, &b))
 		{
 			time_t now;
 
 			time(&now);
 
-			if (now < (time_t)b)	/* Event expired */
+			if (now < b)	/* Event expired */
 			{
 				if (fstat(fileno(fp), &stat_buf) == 0)
 				{
@@ -301,7 +329,7 @@ static int need_refresh()
 					*/
 
 					if (stat_buf.st_mtime >
-					    now - TIMEOUTSOFT/2)
+					    now - get_timeoutsoft()/2)
 					{
 						fclose(fp);	/* Not yet */
 						return (0);
@@ -311,8 +339,9 @@ static int need_refresh()
 		}
 		fclose(fp);
 		unlink(CACHE);
+		return (1);
 	}
-	return (1);
+	return (0);
 }
 
 static void refreshcache(struct PCP *pcp)
@@ -1022,7 +1051,7 @@ void sqpcp_eventparticipants()
 
 void sqpcp_eventfrom()
 {
-	if (access(NOCHANGINGFROM, 0) == 0)
+	if (auth_getoptionenvint("wbnochangingfrom"))
 	{
 		printf("<span class=\"tt\">");
 		print_safe(login_fromhdr());
@@ -1101,7 +1130,7 @@ void sqpcp_summary()
 
 		if (userid && password)
 		{
-			do_login(userid, password, 1);
+			do_pcplogin(userid, password, 1);
 		}
 	}
 
