@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2005 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: pref.c,v 1.25 2003/06/17 15:38:37 mrsam Exp $
+** $Id: pref.c,v 1.28 2006/05/03 02:28:49 mrsam Exp $
 */
 #include	"pref.h"
 #include	"config.h"
@@ -15,7 +15,6 @@
 #include	"mailinglist.h"
 #include	"cgi/cgi.h"
 #include	"pcp/pcp.h"
-#include	"authlib/authstaticlist.h"
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
@@ -31,6 +30,7 @@
 #define FLOWEDTEXT	"NOFLOWEDTEXT"
 #define NOARCHIVE	"NOARCHIVE"
 #define STARTOFWEEK	"STARTOFWEEK"
+#define WIKITEXT		"WIKITEXT"
 
 #define	OLDEST1ST_PREF	"oldest1st"
 #define	FULLHEADERS_PREF "fullheaders"
@@ -48,6 +48,7 @@ int pref_autopurge;
 int pref_noflowedtext;
 int pref_noarchive;
 int pref_startofweek;
+int pref_wikifmt;
 
 char *pref_from=0;
 char *pref_ldap=0;
@@ -101,6 +102,7 @@ char	*q, *r;
 	pref_startofweek=0;
 	pref_autopurge=AUTOPURGE;
 	pref_showhtml=1;
+	pref_wikifmt=0;
 
 	if(pref_from) {
 		free(pref_from);
@@ -126,6 +128,9 @@ char	*q, *r;
 				pref_flagfullheaders=1;
 			if (strcmp(r, NOHTML) == 0)
 				pref_showhtml=0;
+			if (strcmp(r, WIKITEXT) == 0)
+				pref_wikifmt=1;
+
 			if (strncmp(r, SORTORDER, sizeof(SORTORDER)-1) == 0
 				&& r[sizeof(SORTORDER)-1] == '=')
 				pref_flagsortorder=r[sizeof(SORTORDER)];
@@ -263,6 +268,9 @@ char	*q;
 	if (!pref_showhtml)
 		strcat(buf, " " NOHTML);
 
+	if (pref_wikifmt)
+		strcat(buf, " " WIKITEXT);
+
 	p=append_str(buf, FROM, pref_from);
 	q=append_str(p, LDAP, pref_ldap);
 	write_sqconfig(".", CONFIGFILE, q);
@@ -331,13 +339,14 @@ void pref_setprefs()
 		savemailinglists(cgi("mailinglists"));
 	}
 
-	if (*cgi("do.changepwd") && auth_changepass)
+	if (*cgi("do.changepwd"))
 	{
 		int status=1;
 		const	char *p=cgi("newpass");
 		int has_syspwd=0;
 
-		if ( *p && strcmp(p, cgi("newpass2")) == 0)
+		if ( *p && strcmp(p, cgi("newpass2")) == 0
+			&& strlen(p) >= MINPASSLEN)
 		{
 			has_syspwd=
 				login_changepwd(sqwebmail_mailboxid,
