@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2006 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: sqwebmail.c,v 1.125 2004/05/02 03:25:53 mrsam Exp $
+** $Id: sqwebmail.c,v 1.134 2006/05/17 01:42:46 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"sqconfig.h"
@@ -28,8 +28,8 @@
 #include	"maildir/maildiraclt.h"
 #include	"liblock/config.h"
 #include	"liblock/liblock.h"
-#include	"authlib/debug.h"
 #include	"rfc822/rfc822hdr.h"
+#include	"courierauth.h"
 #include	<stdio.h>
 #include	<errno.h>
 #include	<stdlib.h>
@@ -76,11 +76,13 @@
 #define	MD5_INTERNAL
 #include	"md5/md5.h"
 
+#include	<courierauthdebug.h>
 #include	"maildir/maildircache.h"
+#include	"maildir/maildiraclt.h"
+#include	"maildir/maildirnewshared.h"
 #include	"mailfilter.h"
 #include	"numlib/numlib.h"
 #include	"gpglib/gpglib.h"
-#include	"authlib/authstaticlist.h"
 #include	"gpg.h"
 #if	HAVE_CRYPT_H
 #include	<crypt.h>
@@ -148,6 +150,8 @@ static int noimages=0;
 
 time_t	login_time;
 
+extern int nochangepass();
+
 /* Need to cache the following environment variables */
 static const char * const authvars[] = { "AUTHADDR", "AUTHFULLNAME",
 					 "OPTIONS", 0 };
@@ -173,6 +177,19 @@ struct template_stack {
 static struct template_stack *template_stack=NULL;
 
 
+size_t get_timeoutsoft()
+{
+	time_t n=TIMEOUTSOFT;
+	const char *p;
+
+	p=getenv("SQWEBMAIL_TIMEOUTSOFT");
+
+	if (p && *p)
+		n=atoi(p);
+
+	return n;
+}
+
 void fake_exit(int n)
 {
 	maildir_cache_cancel();
@@ -787,7 +804,7 @@ void output_form(const char *formname)
 	pid_t	pid= -1;
 #endif
 
-	noimages= access(NOIMAGES, 0) == 0;
+	noimages= auth_getoptionenvint("wbnoimages");
 
 	f=do_open_form(formname, 1);
 
@@ -865,7 +882,7 @@ void output_form(const char *formname)
 			close(pipefd[1]);
 			execl(GZIP, "gzip", "-c", (char *)0);
 			fprintf(stderr, 
-			       "CRIT: Cannot execute " GZIP ": %s\n",
+			       "ERR: Cannot execute " GZIP ": %s\n",
 			       strerror(errno));
 			exit(1);
 		}
@@ -943,7 +960,7 @@ static void do_output_form_loop(FILE *f)
 				continue;
 			}
 			if (
-			    ( c == '0' && auth_changepass == 0) ||
+			    ( c == '0' && nochangepass()) ||
 			    (c == '1' && strncmp(cgi("folderdir"),
 						 SHARED ".",
 						 sizeof(SHARED)) == 0) ||
@@ -1063,9 +1080,12 @@ static void do_output_form_loop(FILE *f)
 		}
 		else if (strcmp(kw, "d") == 0)
 		{
-			const char *c=cgi("folderdir");
+			const char *f=cgi("folderdir");
+			char *origc, *c;
 			const char *sep="";
 
+			origc=c=folder_fromutf7(f);
+
 			if (*c && strcmp(c, INBOX))
 			{
 				printf(" - ");
@@ -1085,6 +1105,7 @@ static void do_output_form_loop(FILE *f)
 					print_safe(c+1);
 				}
 			}
+			free(origc);
 		}
 		else if (strcmp(kw, "D") == 0)
 		{
@@ -2044,12 +2065,16 @@ char	*cl=http11_best_content_language(HTMLLIBDIR,
 
 static void rename_sent_folder()
 {
-	char buf[128], buf2[200];
+	char buf[128];
+	char yyyymm[128];
+	const char *yyyymmp;
+
 	time_t t;
 	struct tm *tm;
-	struct stat stat_buf;
 	char *pp;
 
+	(void)maildir_create(INBOX "." SENT); /* No matter what */
+
 	time(&t);
 	tm=localtime(&t);
 	if (!tm)
@@ -2063,20 +2088,23 @@ static void rename_sent_folder()
 	else
 		--tm->tm_mon;
 
-	if (strftime (buf, sizeof(buf), "." SENT ".%Y.%m-%b", tm) == 0)
+	if (strftime (yyyymm, sizeof(yyyymm), "%Y%m", tm) == 0)
 		return;
 
-	strcat(strcpy(buf2, "." SENT "/sqwebmail"), buf);
+	if ((yyyymmp=read_sqconfig(".", SENTSTAMP, NULL)) != NULL &&
+	    strcmp(yyyymm, yyyymmp) == 0)
+		return;
 
-	if (stat(buf2, &stat_buf) == 0)
-		return;	/* Already renamed */
+	if (strftime (buf, sizeof(buf), "." SENT ".%Y.%m-%b", tm) == 0)
+		return;
 
 	pp=folder_toutf7(buf);
 
 	rename("." SENT, pp);
 	free(pp);
 	(void)maildir_create(INBOX "." SENT);
-	close(open(buf2, O_RDWR|O_CREAT, 0600));
+
+	write_sqconfig(".", SENTSTAMP, yyyymm);
 }
 
 static int valid_redirect();
@@ -2141,7 +2169,7 @@ int main(int argc, char **argv)
 	const char *prefork;
 	int n;
 
-	auth_debug_login_init();
+	courier_authdebug_login_init();
 
 	if (argc > 1)
 		usage();
@@ -2438,7 +2466,7 @@ static void force_read(int cfd, char *p, size_t l)
 		if (m <= 0)
 		{
 			fprintf(stderr,
-			       "CRIT: socket closed while reading"
+			       "WARN: socket closed while reading"
 			       " environment.\n");
 			exit(0);
 		}
@@ -2665,12 +2693,7 @@ time_t	timeouthard=TIMEOUTHARD;
 	const char	*p;
 	time_t	last_time, current_time;
 	char	*q;
-	time_t	timeoutsoft=TIMEOUTSOFT;
-
-		p=getenv("SQWEBMAIL_TIMEOUTSOFT");
-
-		if (p && *p)
-			timeoutsoft=atoi(p);
+	time_t	timeoutsoft=get_timeoutsoft();
 
 		/* Logging into the mailbox */
 
@@ -2825,6 +2848,23 @@ time_t	timeouthard=TIMEOUTHARD;
 
 		pref_init();
 		(void)sqpcp_loggedin();
+		if (auth_getoptionenvint("disableshared"))
+		{
+			maildir_acl_disabled=1;
+			maildir_newshared_disabled=1;
+		}
+
+		if (strcmp(cgi("form"), "empty"))
+		{
+			if (*cgi("refresh"))
+			{
+				printf("Refresh: %ld; URL=",
+				       (long)get_timeoutsoft()/2);
+				output_scriptptrget();
+				printf("&empty=1&refresh=1\n");
+			}
+		}
+
 		output_user_form(cgi("form"));
 		free(pi);
 	}
@@ -2859,7 +2899,6 @@ time_t	timeouthard=TIMEOUTHARD;
 		const char *mailboxid;
 		const char *u2=cgi("logindomain");
 		char	*ubuf=malloc(strlen(u)+strlen(u2)+2);
-		int can_changepwd;
 
 			strcpy(ubuf, u);
 			if (*u2)
@@ -2867,7 +2906,7 @@ time_t	timeouthard=TIMEOUTHARD;
 
 			maildir_cache_start();
 
-			if (*p && (mailboxid=login(ubuf, p, &can_changepwd))
+			if (*p && (mailboxid=do_login(ubuf, p, ip_addr))
 			    != 0)
 			{
 				char	*q;
@@ -2908,7 +2947,7 @@ time_t	timeouthard=TIMEOUTHARD;
 					if (getcwd(buf, sizeof(buf)-1) == 0)
 					{
 						fprintf(stderr,
-						       "ERR: getcwd() failed: %s\n",strerror(errno));
+						       "CRIT: getcwd() failed: %s\n",strerror(errno));
 						fake_exit(1);
 					} /* oops */
 
@@ -2956,8 +2995,9 @@ time_t	timeouthard=TIMEOUTHARD;
 				sqpcp_login(ubuf, p);
 				maildir_acl_reset(".");
 
-				http_redirect_argss("&form=folders", "", "");
-				/* output_form("folders.html"); */
+				http_redirect_argss(*cgi("inpublic") ?
+						    "&form=folders":
+						    "&form=refreshfr", "", "");
 				free(ubuf);
 				return(0);
 			}
@@ -2981,7 +3021,9 @@ time_t	timeouthard=TIMEOUTHARD;
 			output_form("login.html");	/* Main frame */
 		else
 		if ( *cgi("empty") == '1')
+		{
 			output_form("empty.html");	/* Minor frameset */
+		}
 
 /*
 ** Apparently we can't show just SCRIPT NAME as our frameset due to some
